@@ -208,8 +208,14 @@ _init_common_subsystems (gpg_err_source_t errsource, int *argcp, char ***argvp)
   gettext_use_utf8 (1);
   if (!SetConsoleCP (CP_UTF8) || !SetConsoleOutputCP (CP_UTF8))
     {
-      log_info ("SetConsoleCP failed: %s\n", w32_strerror (-1));
-      log_info ("Warning: Garbled console data possible\n");
+      /* Don't show the error if the program does not have a console.
+       * This is for example the case for daemons.  */
+      int rc = GetLastError ();
+      if (rc != ERROR_INVALID_HANDLE)
+        {
+          log_info ("SetConsoleCP failed: %s\n", w32_strerror (rc));
+          log_info ("Warning: Garbled console data possible\n");
+        }
     }
 #endif
 
@@ -224,10 +230,10 @@ _init_common_subsystems (gpg_err_source_t errsource, int *argcp, char ***argvp)
   }
 
   /* --version et al shall use estream as well.  */
-  gnupg_set_usage_outfnc (writestring_via_estream);
+  gpgrt_set_usage_outfnc (writestring_via_estream);
 
   /* Register our string mapper with gpgrt.  */
-  gnupg_set_fixed_string_mapper (map_static_macro_string);
+  gpgrt_set_fixed_string_mapper (map_static_macro_string);
 
   /* Logging shall use the standard socket directory as fallback.  */
   log_set_socket_dir_cb (gnupg_socketdir);
@@ -331,7 +337,7 @@ prepare_w32_commandline (int *r_argc, char ***r_argv)
   const char *s;
   int i, globing, itemsalloced;
 
-  s = strusage (95);
+  s = gpgrt_strusage (95);
   globing = (s && *s == '1');
 
   wcmdline = GetCommandLineW ();
