@@ -106,7 +106,6 @@ enum
     IOBUFCTRL_FLUSH     = 4,
     IOBUFCTRL_DESC	= 5,
     IOBUFCTRL_CANCEL    = 6,
-    IOBUFCTRL_PEEK      = 7,
     IOBUFCTRL_USER	= 16
   };
 
@@ -117,8 +116,7 @@ typedef enum
     IOBUF_IOCTL_KEEP_OPEN        = 1, /* Uses intval.  */
     IOBUF_IOCTL_INVALIDATE_CACHE = 2, /* Uses ptrval.  */
     IOBUF_IOCTL_NO_CACHE         = 3, /* Uses intval.  */
-    IOBUF_IOCTL_FSYNC            = 4, /* Uses ptrval.  */
-    IOBUF_IOCTL_PEEK             = 5  /* Uses intval and ptrval.  */
+    IOBUF_IOCTL_FSYNC            = 4  /* Uses ptrval.  */
   } iobuf_ioctl_t;
 
 enum iobuf_use
@@ -204,26 +202,6 @@ struct iobuf_struct
     byte *buf;
   } d;
 
-  /* A external drain buffer for reading/writting data skipping internal
-     draint buffer D.BUF.  This allows zerocopy operation reducing
-     processing overhead across filter stack.
-
-     Used when by iobuf_read/iobuf_write when internal buffer has been
-     depleted and remaining external buffer length is large enough.
-   */
-  struct
-  {
-    /* The external buffer provided by iobuf_read/iobuf_write caller. */
-    byte *buf;
-    /* The number of bytes in the external buffer. */
-    size_t len;
-    /* The number of bytes that were consumed from the external buffer. */
-    size_t used;
-    /* Gives hint for processing that the external buffer is preferred and
-       that internal buffer should be consumed early. */
-    int preferred;
-  } e_d;
-
   /* When FILTER is called to read some data, it may read some data
      and then return EOF.  We can't return the EOF immediately.
      Instead, we note that we observed the EOF and when the buffer is
@@ -584,8 +562,12 @@ size_t iobuf_temp_to_buffer (iobuf_t a, byte * buffer, size_t buflen);
 size_t iobuf_copy (iobuf_t dest, iobuf_t source);
 
 /* Return the size of any underlying file.  This only works with
-   file_filter based pipelines.  */
-uint64_t iobuf_get_filelength (iobuf_t a);
+   file_filter based pipelines.
+
+   On Win32, it is sometimes not possible to determine the size of
+   files larger than 4GB.  In this case, *OVERFLOW (if not NULL) is
+   set to 1.  Otherwise, *OVERFLOW is set to 0.  */
+off_t iobuf_get_filelength (iobuf_t a, int *overflow);
 #define IOBUF_FILELENGTH_LIMIT 0xffffffff
 
 /* Return the file descriptor designating the underlying file.  This
@@ -625,9 +607,6 @@ void iobuf_set_partial_body_length_mode (iobuf_t a, size_t len);
    from the following filter (which may or may not return EOF).  */
 void iobuf_skip_rest (iobuf_t a, unsigned long n, int partial);
 
-/* Check if the file is compressed, by peeking the iobuf.  */
-int is_file_compressed (iobuf_t inp);
-
 #define iobuf_where(a)	"[don't know]"
 
 /* Each time a filter is allocated (via iobuf_alloc()), a
