@@ -1,7 +1,6 @@
 /* decrypt-data.c - Decrypt an encrypted data packet
  * Copyright (C) 1998-2001, 2005-2006, 2009 Free Software Foundation, Inc.
  * Copyright (C) 1998-2001, 2005-2006, 2009, 2018 Werner Koch
- * Copyright (C) 2020 g10 Code GmbH
  *
  * This file is part of GnuPG.
  *
@@ -44,7 +43,7 @@ static int decode_filter ( void *opaque, int control, IOBUF a,
 /* Our context object.  */
 struct decode_filter_context_s
 {
-  /* Recounter (max value is 2).  We need it because we do not know
+  /* Redcounter (max value is 2).  We need it because we do not know
    * whether the iobuf or the outer control code frees this object
    * first.  */
   int  refcount;
@@ -73,9 +72,6 @@ struct decode_filter_context_s
    *   3 = premature EOF (general)       */
   unsigned int eof_seen : 2;
 
-  /* Flag to convey an error from aead_checktag.  */
-  unsigned int checktag_failed : 1;
-
   /* The actually used cipher algo for AEAD.  */
   byte cipher_algo;
 
@@ -209,8 +205,6 @@ aead_checktag (decode_filter_ctx_t dfx, int final, const void *tagbuf)
     {
       log_error ("gcry_cipher_checktag%s failed: %s\n",
                  final? " (final)":"", gpg_strerror (err));
-      write_status_error ("aead_checktag", err);
-      dfx->checktag_failed = 1;
       return err;
     }
   if (DBG_FILTER)
@@ -220,14 +214,10 @@ aead_checktag (decode_filter_ctx_t dfx, int final, const void *tagbuf)
 
 
 /****************
- * Decrypt the data, specified by ED with the key DEK.  On return
- * COMPLIANCE_ERROR is set to true iff the decryption can claim that
- * it was compliant in the current mode; otherwise this flag is set to
- * false.
+ * Decrypt the data, specified by ED with the key DEK.
  */
 int
-decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek,
-              int *compliance_error)
+decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek)
 {
   decode_filter_ctx_t dfx;
   enum gcry_cipher_modes ciphermode;
@@ -235,10 +225,8 @@ decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek,
   byte *p;
   int rc=0, c, i;
   byte temp[32];
-  unsigned blocksize;
-  unsigned nprefix;
-
-  *compliance_error = 0;
+  unsigned int blocksize;
+  unsigned int nprefix;
 
   dfx = xtrycalloc (1, sizeof *dfx);
   if (!dfx)
@@ -248,8 +236,10 @@ decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek,
   if ( opt.verbose && !dek->algo_info_printed )
     {
       if (!openpgp_cipher_test_algo (dek->algo))
-        log_info (_("%s encrypted data\n"),
-                  openpgp_cipher_algo_mode_name (dek->algo, ed->aead_algo));
+        log_info (_("%s.%s encrypted data\n"),
+                  openpgp_cipher_algo_name (dek->algo),
+                  ed->aead_algo? openpgp_aead_algo_name (ed->aead_algo)
+                  /**/         : "CFB");
       else
         log_info (_("encrypted with unknown algorithm %d\n"), dek->algo );
       dek->algo_info_printed = 1;
@@ -269,24 +259,18 @@ decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek,
   if (!gnupg_cipher_is_allowed (opt.compliance, 0, dek->algo, ciphermode))
     {
       log_error (_("cipher algorithm '%s' may not be used in %s mode\n"),
-		 openpgp_cipher_algo_mode_name (dek->algo,ed->aead_algo),
+		 openpgp_cipher_algo_name (dek->algo),
 		 gnupg_compliance_option_string (opt.compliance));
-      *compliance_error = 1;
-      if (opt.flags.require_compliance)
-        {
-          /* We fail early in this case because it does not make sense
-           * to first decrypt everything.  */
-          rc = gpg_error (GPG_ERR_CIPHER_ALGO);
-          goto leave;
-        }
+      rc = gpg_error (GPG_ERR_CIPHER_ALGO);
+      goto leave;
     }
 
   write_status_printf (STATUS_DECRYPTION_INFO, "%d %d %d",
-                       ed->mdc_method, dek->algo, 0);
+                       ed->mdc_method, dek->algo, ed->aead_algo);
 
   if (opt.show_session_key)
     {
-      char numbuf[25];
+      char numbuf[30];
       char *hexbuf;
 
       if (ed->aead_algo)
@@ -429,18 +413,17 @@ decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek,
         {
           log_info (_("WARNING: message was encrypted with"
                       " a weak key in the symmetric cipher.\n"));
-          rc = 0;
+          rc=0;
         }
       else if (rc)
         {
-          log_error("key setup failed: %s\n", gpg_strerror (rc) );
+          log_error ("key setup failed: %s\n", gpg_strerror (rc) );
           goto leave;
         }
 
       if (!ed->buf)
         {
           log_error (_("problem handling encrypted packet\n"));
-          rc = gpg_error (GPG_ERR_INV_PACKET);
           goto leave;
         }
 
@@ -483,7 +466,6 @@ decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek,
   dfx->refcount++;
   dfx->partial = !!ed->is_partial;
   dfx->length = ed->len;
-  dfx->checktag_failed = 0;
   if (ed->aead_algo)
     iobuf_push_filter ( ed->buf, aead_decode_filter, dfx );
   else if (ed->mdc_method)
@@ -499,11 +481,9 @@ decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek,
       rc = get_output_file ("", 0, ed->buf, &filename, &fp);
       if (! rc)
         {
-          iobuf_t output = iobuf_esopen (fp, "w", 0);
+          iobuf_t output = iobuf_esopen (fp, "w", 0, 0);
           armor_filter_context_t *afx = NULL;
 
-	  es_setbuf (fp, NULL);
-
           if (opt.armor)
             {
               afx = new_armor_context ();
@@ -529,10 +509,6 @@ decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek,
   ed->buf = NULL;
   if (dfx->eof_seen > 1 )
     rc = gpg_error (GPG_ERR_INV_PACKET);
-  else if (dfx->checktag_failed)
-    {
-      rc = gpg_error (GPG_ERR_BAD_SIGNATURE);
-    }
   else if ( ed->mdc_method )
     {
       /* We used to let parse-packet.c handle the MDC packet but this
@@ -561,8 +537,8 @@ decrypt_data (ctrl_t ctrl, void *procctx, PKT_encrypted *ed, DEK *dek,
           || datalen != 20
           || memcmp (gcry_md_read (dfx->mdc_hash, 0), dfx->holdback+2, datalen))
         rc = gpg_error (GPG_ERR_BAD_SIGNATURE);
-      /* log_printhex(dfx->holdback, 22, "MDC message:"); */
-      /* log_printhex(gcry_md_read (dfx->mdc_hash,0), datalen, "MDC calc:"); */
+      /* log_printhex("MDC message:", dfx->holdback, 22); */
+      /* log_printhex("MDC calc:", gcry_md_read (dfx->mdc_hash,0), datalen); */
     }
 
  leave:
@@ -936,7 +912,7 @@ mdc_decode_filter (void *opaque, int control, IOBUF a,
       n = fill_buffer (dfx, a, buf, 44, 22);
       if (n == 44)
         {
-          /* We have enough stuff - flush the deferred stuff.  */
+          /* We have enough stuff - flush the holdback buffer.  */
           if ( !dfx->holdbacklen )  /* First time. */
             {
               memcpy (buf, buf+22, 22);
@@ -946,6 +922,7 @@ mdc_decode_filter (void *opaque, int control, IOBUF a,
             {
               memcpy (buf, dfx->holdback, 22);
 	    }
+
           /* Fill up the buffer. */
           n = fill_buffer (dfx, a, buf, size, n);
 
@@ -955,7 +932,7 @@ mdc_decode_filter (void *opaque, int control, IOBUF a,
           memcpy (dfx->holdback, buf+n, 22 );
           dfx->holdbacklen = 22;
 	}
-      else if ( !dfx->holdbacklen )  /* EOF seen but empty holdback buffer. */
+      else if ( !dfx->holdbacklen ) /* EOF seen but empty holdback. */
         {
           /* This is bad because it means an incomplete hash. */
           n -= 22;
