@@ -155,7 +155,6 @@ int get_active_connection_count (void);
 
 /*-- app.c --*/
 int scd_update_reader_status_file (void);
-void app_wait (void);
-gpg_error_t app_send_devinfo (ctrl_t ctrl);
+gpg_error_t app_send_devinfo (ctrl_t ctrl, int keep_looping);
 
 #endif /*SCDAEMON_H*/
