@@ -213,6 +213,13 @@ static struct
   { "1.2.840.113583.1.1.5",     "pdfAuthenticDocumentsTrust", OID_FLAG_KP },
   { "1.3.6.1.4.1.6449.1.3.5.2", "comodoCertifiedDeliveryService", OID_FLAG_KP },
 
+  /* ARRL */
+  { "1.3.6.1.4.1.12348.1.1",    "lotw-callsign" },
+  { "1.3.6.1.4.1.12348.1.2",    "lotw-qso-first-date" },
+  { "1.3.6.1.4.1.12348.1.3",    "lotw-qso-end-date" },
+  { "1.3.6.1.4.1.12348.1.4",    "lotw-dxcc-entity" },
+  /* { "1.3.6.1.4.1.12348.1.5",    "lotw-fixme" }, */
+
   { NULL }
 };
 
@@ -1453,6 +1460,7 @@ list_cert_chain (ctrl_t ctrl, KEYDB_HANDLE hd,
                  estream_t fp, int with_validation)
 {
   ksba_cert_t next = NULL;
+  int depth = 0;
 
   if (raw_mode)
     list_cert_raw (ctrl, hd, cert, fp, 0, with_validation);
@@ -1461,8 +1469,13 @@ list_cert_chain (ctrl_t ctrl, KEYDB_HANDLE hd,
   ksba_cert_ref (cert);
   while (!gpgsm_walk_cert_chain (ctrl, cert, &next))
     {
-      ksba_cert_release (cert);
       es_fputs ("Certified by\n", fp);
+      if (++depth > 50)
+        {
+          es_fputs (_("certificate chain too long\n"), fp);
+          break;
+        }
+      ksba_cert_release (cert);
       if (raw_mode)
         list_cert_raw (ctrl, hd, next, fp, 0, with_validation);
       else
