@@ -1,6 +1,6 @@
 /* gpgconf.c - Configuration utility for GnuPG
  * Copyright (C) 2003, 2007, 2009, 2011 Free Software Foundation, Inc.
- * Copyright (C) 2016, 2020 g10 Code GmbH.
+ * Copyright (C) 2016 g10 Code GmbH.
  *
  * This file is part of GnuPG.
  *
@@ -20,6 +20,7 @@
  */
 
 #include <config.h>
+
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -34,6 +35,9 @@
 #include "../common/status.h"
 #include "../common/exechelp.h"
 
+#ifdef HAVE_W32_SYSTEM
+#include <windows.h>
+#endif
 
 /* Constants to identify the commands and options. */
 enum cmd_and_opt_values
@@ -46,17 +50,12 @@ enum cmd_and_opt_values
     oRuntime    = 'r',
     oComponent  = 'c',
     oNull       = '0',
-    aListDirs   = 'L',
-    aKill       = 'K',
-    aReload     = 'R',
-    aShowVersions = 'V',
-    aShowConfigs  = 'X',
-
     oNoVerbose	= 500,
     oHomedir,
     oBuilddir,
     oStatusFD,
     oShowSocket,
+    oChUid,
 
     aListComponents,
     aCheckPrograms,
@@ -67,63 +66,68 @@ enum cmd_and_opt_values
     aListConfig,
     aCheckConfig,
     aQuerySWDB,
+    aListDirs,
     aLaunch,
+    aKill,
     aCreateSocketDir,
     aRemoveSocketDir,
-    aApplyProfile
+    aApplyProfile,
+    aReload,
+    aShowVersions,
+    aShowConfigs,
+    aShowCodepages
   };
 
 
 /* The list of commands and options. */
-static ARGPARSE_OPTS opts[] =
+static gpgrt_opt_t opts[] =
   {
-    ARGPARSE_group (300, N_("@Commands:\n ")),
+    { 300, NULL, 0, N_("@Commands:\n ") },
 
-    ARGPARSE_c (aListComponents, "list-components", N_("list all components")),
-    ARGPARSE_c (aCheckPrograms, "check-programs", N_("check all programs")),
-    ARGPARSE_c (aListOptions, "list-options", N_("|COMPONENT|list options")),
-    ARGPARSE_c (aChangeOptions, "change-options",
-                N_("|COMPONENT|change options")),
-    ARGPARSE_c (aCheckOptions, "check-options", N_("|COMPONENT|check options")),
-    ARGPARSE_c (aApplyDefaults, "apply-defaults",
-                N_("apply global default values")),
-    ARGPARSE_c (aApplyProfile, "apply-profile",
-                N_("|FILE|update configuration files using FILE")),
-    ARGPARSE_c (aListDirs, "list-dirs",
-                N_("get the configuration directories for @GPGCONF@")),
-    ARGPARSE_c (aListConfig, "list-config",
-                N_("list global configuration file")),
-    ARGPARSE_c (aCheckConfig, "check-config",
-                N_("check global configuration file")),
-    ARGPARSE_c (aQuerySWDB, "query-swdb",
-                N_("query the software version database")),
-    ARGPARSE_c (aReload, "reload", N_("reload all or a given component")),
-    ARGPARSE_c (aLaunch, "launch", N_("launch a given component")),
-    ARGPARSE_c (aKill, "kill", N_("kill a given component")),
-    ARGPARSE_c (aCreateSocketDir, "create-socketdir", "@"),
-    ARGPARSE_c (aRemoveSocketDir, "remove-socketdir", "@"),
-    ARGPARSE_c (aShowVersions, "show-versions", ""),
-    ARGPARSE_c (aShowConfigs,  "show-configs", ""),
+    { aListComponents, "list-components", 256, N_("list all components") },
+    { aCheckPrograms, "check-programs", 256, N_("check all programs") },
+    { aListOptions, "list-options", 256, N_("|COMPONENT|list options") },
+    { aChangeOptions, "change-options", 256, N_("|COMPONENT|change options") },
+    { aCheckOptions, "check-options", 256, N_("|COMPONENT|check options") },
+    { aApplyDefaults, "apply-defaults", 256,
+      N_("apply global default values") },
+    { aApplyProfile, "apply-profile", 256,
+      N_("|FILE|update configuration files using FILE") },
+    { aListDirs, "list-dirs", 256,
+      N_("get the configuration directories for @GPGCONF@") },
+    { aListConfig,   "list-config", 256,
+      N_("list global configuration file") },
+    { aCheckConfig,   "check-config", 256,
+      N_("check global configuration file") },
+    { aQuerySWDB,     "query-swdb", 256,
+      N_("query the software version database") },
+    { aReload,        "reload", 256, N_("reload all or a given component")},
+    { aLaunch,        "launch", 256, N_("launch a given component")},
+    { aKill,          "kill", 256,   N_("kill a given component")},
+    { aCreateSocketDir, "create-socketdir", 256, "@"},
+    { aRemoveSocketDir, "remove-socketdir", 256, "@"},
+    ARGPARSE_c (aShowVersions, "show-versions", "@"),
+    ARGPARSE_c (aShowConfigs,  "show-configs", "@"),
+    ARGPARSE_c (aShowCodepages, "show-codepages", "@"),
 
-    ARGPARSE_header (NULL, N_("@\nOptions:\n ")),
+    { 301, NULL, 0, N_("@\nOptions:\n ") },
 
-    ARGPARSE_s_s (oOutput, "output", N_("use as output file")),
-    ARGPARSE_s_n (oVerbose, "verbose", N_("verbose")),
-    ARGPARSE_s_n (oQuiet, "quiet", N_("quiet")),
-    ARGPARSE_s_n (oDryRun, "dry-run", N_("do not make any changes")),
-    ARGPARSE_s_n (oRuntime, "runtime",
-                  N_("activate changes at runtime, if possible")),
+    { oOutput, "output",    2, N_("use as output file") },
+    { oVerbose, "verbose",  0, N_("verbose") },
+    { oQuiet, "quiet",      0, N_("quiet") },
+    { oDryRun, "dry-run",   0, N_("do not make any changes") },
+    { oRuntime, "runtime",  0, N_("activate changes at runtime, if possible") },
     ARGPARSE_s_i (oStatusFD, "status-fd",
                   N_("|FD|write status info to this FD")),
-
     /* hidden options */
-    ARGPARSE_s_s (oHomedir, "homedir", "@"),
-    ARGPARSE_s_s (oBuilddir, "build-prefix", "@"),
-    ARGPARSE_s_n (oNull, "null", "@"),
-    ARGPARSE_s_n (oNoVerbose, "no-verbose", "@"),
+    { oHomedir, "homedir", 2, "@" },
+    { oBuilddir, "build-prefix", 2, "@" },
+    { oNull, "null", 0, "@" },
+    { oNoVerbose, "no-verbose",  0, "@"},
     ARGPARSE_s_n (oShowSocket, "show-socket", "@"),
+    ARGPARSE_s_s (oChUid, "chuid", "@"),
 
-    ARGPARSE_end ()
+    ARGPARSE_end(),
   };
 
 
@@ -247,10 +251,10 @@ gpgconf_write_status (int no, const char *format, ...)
 
 
 static void
-list_dirs (estream_t fp, char **names, int show_config_mode)
+list_dirs (estream_t fp, char **names, int special)
 {
   static struct {
-    const char *name; /* If NULL only a file check will be done.  */
+    const char *name;
     const char *(*fnc)(void);
     const char *extra;
   } list[] = {
@@ -262,20 +266,18 @@ list_dirs (estream_t fp, char **names, int show_config_mode)
     { "localedir",          gnupg_localedir,  NULL },
     { "socketdir",          gnupg_socketdir,  NULL },
     { "dirmngr-socket",     dirmngr_socket_name, NULL,},
+    { "keyboxd-socket",     keyboxd_socket_name, NULL,},
     { "agent-ssh-socket",   gnupg_socketdir,  GPG_AGENT_SSH_SOCK_NAME },
     { "agent-extra-socket", gnupg_socketdir,  GPG_AGENT_EXTRA_SOCK_NAME },
     { "agent-browser-socket",gnupg_socketdir, GPG_AGENT_BROWSER_SOCK_NAME },
     { "agent-socket",       gnupg_socketdir,  GPG_AGENT_SOCK_NAME },
-    { NULL,                 gnupg_socketdir,  "S.uiserver" },
     { "homedir",            gnupg_homedir,    NULL }
   };
   int idx, j;
   char *tmp;
   const char *s;
-  gpg_error_t err;
 
-  if (show_config_mode)
-    es_fprintf (fp, "#+begin_example\n");
+
   for (idx = 0; idx < DIM (list); idx++)
     {
       s = list[idx].fnc ();
@@ -286,45 +288,20 @@ list_dirs (estream_t fp, char **names, int show_config_mode)
         }
       else
         tmp = NULL;
-
-      if (!list[idx].name)
-        ;
-      else if (!names)
-        es_fprintf (fp, "%s%s:%s\n", show_config_mode? "  ":"",
-                    list[idx].name, gc_percent_escape (s));
+      if (!names)
+        es_fprintf (fp, "%s:%s\n", list[idx].name, gc_percent_escape (s));
       else
         {
           for (j=0; names[j]; j++)
             if (!strcmp (names[j], list[idx].name))
               {
-                if (show_config_mode)
-                  es_fputs ("  ", fp);
                 es_fputs (s, fp);
                 es_putc (opt.null? '\0':'\n', fp);
               }
         }
 
-      /* In show config mode check that the socket files are accessible.  */
-      if (list[idx].extra && show_config_mode)
-        {
-          estream_t tmpfp;
-
-          tmpfp = es_fopen (s, "rb");
-          if (tmpfp)
-            es_fclose (tmpfp);  /* All fine - we can read that file.  */
-          else if ((err=gpg_error_from_syserror ()) == GPG_ERR_ENOENT
-                   || err == GPG_ERR_ENXIO)
-            ; /* No such file/ No such device or address - this is okay.  */
-          else
-            es_fprintf (fp,
-                        "# Warning: error reading existing file '%s': %s\n",
-                        s, gpg_strerror (err));
-        }
-
       xfree (tmp);
     }
-  if (show_config_mode)
-    es_fprintf (fp, "#+end_example\n");
 
 
 #ifdef HAVE_W32_SYSTEM
@@ -353,11 +330,11 @@ list_dirs (estream_t fp, char **names, int show_config_mode)
         }
 
       es_fflush (fp);
-      if (show_config_mode)
+      if (special)
         es_fprintf (fp, "\n"
-                    "Note: homedir taken from registry key %s%s\\%s:%s\n"
+                    "### Note: homedir taken from registry key %s%s\\%s:%s\n"
                     "\n",
-                    hkcu?" HKCU":"", hklm?" HKLM":"",
+                    hkcu?"HKCU":"", hklm?"HKLM":"",
                     GNUPG_REGISTRY_DIR, "HomeDir");
       else
         log_info ("Warning: homedir taken from registry key (%s:%s) in%s%s\n",
@@ -371,9 +348,9 @@ list_dirs (estream_t fp, char **names, int show_config_mode)
     {
       xfree (tmp);
       es_fflush (fp);
-      if (show_config_mode)
+      if (special)
         es_fprintf (fp, "\n"
-                    "Note: registry key %s without value in HKCU or HKLM\n"
+                    "### Note: registry %s without value in HKCU or HKLM\n"
                     "\n", GNUPG_REGISTRY_DIR);
       else
         log_info ("Warning: registry key (%s) without value in HKCU or HKLM\n",
@@ -381,7 +358,7 @@ list_dirs (estream_t fp, char **names, int show_config_mode)
     }
 
 #else /*!HAVE_W32_SYSTEM*/
-  (void)show_config_mode;
+  (void)special;
 #endif /*!HAVE_W32_SYSTEM*/
 }
 
@@ -425,7 +402,7 @@ valid_swdb_name_p (const char *name)
  *            Common codes seen:
  *            GPG_ERR_TOO_OLD :: The SWDB file is to old to be used.
  *            GPG_ERR_ENOENT  :: The SWDB file is not available.
- *            GPG_ERR_BAD_SIGNATURE :: Currupted SWDB file.
+ *            GPG_ERR_BAD_SIGNATURE :: Corrupted SWDB file.
  * filedate:: Date of the swdb file (yyyymmddThhmmss)
  * verified:: Date we checked the validity of the file (yyyyymmddThhmmss)
  * version :: The version string from the swdb.
@@ -446,7 +423,7 @@ query_swdb (estream_t out, const char *name, const char *current_version)
   size_t length_of_line = 0;
   size_t  maxlen;
   ssize_t len;
-  char *fields[2];
+  const char *fields[2];
   char *p;
   gnupg_isotime_t filedate = {0};
   gnupg_isotime_t verified = {0};
@@ -631,17 +608,18 @@ int
 main (int argc, char **argv)
 {
   gpg_error_t err;
-  ARGPARSE_ARGS pargs;
+  gpgrt_argparse_t pargs;
   const char *fname;
   int no_more_options = 0;
   enum cmd_and_opt_values cmd = 0;
   estream_t outfp = NULL;
   int show_socket = 0;
+  const char *changeuser = NULL;
 
   early_system_init ();
   gnupg_reopen_std (GPGCONF_NAME);
-  set_strusage (my_strusage);
-  log_set_prefix (GPGCONF_NAME, GPGRT_LOG_WITH_PREFIX|GPGRT_LOG_NO_REGISTRY);
+  gpgrt_set_strusage (my_strusage);
+  log_set_prefix (GPGCONF_NAME, GPGRT_LOG_WITH_PREFIX);
 
   /* Make sure that our subsystems are ready.  */
   i18n_init();
@@ -652,7 +630,7 @@ main (int argc, char **argv)
   pargs.argc  = &argc;
   pargs.argv  = &argv;
   pargs.flags = ARGPARSE_FLAG_KEEP;
-  while (!no_more_options && gnupg_argparse (NULL, &pargs, opts))
+  while (!no_more_options && gpgrt_argparse (NULL, &pargs, opts))
     {
       switch (pargs.r_opt)
         {
@@ -669,6 +647,7 @@ main (int argc, char **argv)
           set_status_fd (translate_sys2libc_fd_int (pargs.r.ret_int, 1));
           break;
         case oShowSocket: show_socket = 1; break;
+        case oChUid:      changeuser = pargs.r.ret_str; break;
 
 	case aListDirs:
         case aListComponents:
@@ -688,13 +667,15 @@ main (int argc, char **argv)
         case aRemoveSocketDir:
         case aShowVersions:
         case aShowConfigs:
+        case aShowCodepages:
 	  cmd = pargs.r_opt;
 	  break;
 
-        default: pargs.err = ARGPARSE_PRINT_ERROR; break;
+        default: pargs.err = 2; break;
 	}
     }
-  gnupg_argparse (NULL, &pargs, NULL);  /* Release internal state.  */
+
+  gpgrt_argparse (NULL, &pargs, NULL);  /* Release internal state.  */
 
   if (log_get_errorcount (0))
     gpgconf_failure (GPG_ERR_USER_2);
@@ -711,11 +692,15 @@ main (int argc, char **argv)
 
   fname = argc ? *argv : NULL;
 
+  /* If requested switch to the requested user or die.  */
+  if (changeuser && (err = gnupg_chuid (changeuser, 0)))
+    gpgconf_failure (err);
+
   /* Set the configuraton directories for use by gpgrt_argparser.  We
    * don't have a configuration file for this program but we have code
    * which reads the component's config files.  */
-  gnupg_set_confdir (GNUPG_CONFDIR_SYS, gnupg_sysconfdir ());
-  gnupg_set_confdir (GNUPG_CONFDIR_USER, gnupg_homedir ());
+  gpgrt_set_confdir (GPGRT_CONFDIR_SYS, gnupg_sysconfdir ());
+  gpgrt_set_confdir (GPGRT_CONFDIR_USER, gnupg_homedir ());
 
   switch (cmd)
     {
@@ -811,6 +796,8 @@ main (int argc, char **argv)
                     names[0] = "agent-socket";
                   else if (idx == GC_COMPONENT_DIRMNGR)
                     names[0] = "dirmngr-socket";
+                  else if (idx == GC_COMPONENT_KEYBOXD)
+                    names[0] = "keyboxd-socket";
                   else
                     names[0] = NULL;
                   names[1] = NULL;
@@ -962,7 +949,7 @@ main (int argc, char **argv)
           ;
         else if (gnupg_rmdir (socketdir))
           {
-            /* If the director is not empty we first try to delet
+            /* If the director is not empty we first try to delete
              * socket files.  */
             err = gpg_error_from_syserror ();
             if (gpg_err_code (err) == GPG_ERR_ENOTEMPTY
@@ -974,6 +961,7 @@ main (int argc, char **argv)
                   GPG_AGENT_BROWSER_SOCK_NAME,
                   GPG_AGENT_SSH_SOCK_NAME,
                   SCDAEMON_SOCK_NAME,
+                  KEYBOXD_SOCK_NAME,
                   DIRMNGR_SOCK_NAME
                 };
                 int i;
@@ -1016,6 +1004,22 @@ main (int argc, char **argv)
       }
       break;
 
+    case aShowCodepages:
+#ifdef HAVE_W32_SYSTEM
+      {
+        get_outfp (&outfp);
+        if (GetConsoleCP () != GetConsoleOutputCP ())
+          es_fprintf (outfp, "Console: CP%u/CP%u\n",
+                      GetConsoleCP (), GetConsoleOutputCP ());
+        else
+          es_fprintf (outfp, "Console: CP%u\n", GetConsoleCP ());
+        es_fprintf (outfp, "ANSI: CP%u\n", GetACP ());
+        es_fprintf (outfp, "OEM: CP%u\n", GetOEMCP ());
+      }
+#endif
+      break;
+
+
     }
 
   if (outfp != es_stdout)
@@ -1074,71 +1078,8 @@ get_revision_from_blurb (const char *blurb, int *r_len)
 static void
 show_version_gnupg (estream_t fp, const char *prefix)
 {
-  char *fname, *p, *p0;
-  size_t n;
-  estream_t verfp;
-  char *line = NULL;
-  size_t line_len = 0;
-  ssize_t length;
-
   es_fprintf (fp, "%s%sGnuPG %s (%s)\n%s%s\n", prefix, *prefix?"":"* ",
-              strusage (13), BUILD_REVISION, prefix, strusage (17));
-
-  /* Show the GnuPG VS-Desktop version in --show-configs mode  */
-  if (prefix && *prefix)
-    {
-      fname = make_filename (gnupg_bindir (), NULL);
-      n = strlen (fname);
-      if (n > 10 && (!ascii_strcasecmp (fname + n - 10, "/GnuPG/bin")
-                     || !ascii_strcasecmp (fname + n - 10, "\\GnuPG\\bin")))
-        {
-          /* Append VERSION to the ../../ direcory.  Note that VERSION
-           * is only 7 bytes and thus fits.  */
-          strcpy (fname + n - 9, "VERSION");
-          verfp = es_fopen (fname, "r");
-          if (!verfp)
-            es_fprintf (fp, "%s[VERSION file not found]\n", prefix);
-          else
-            {
-              int lnr = 0;
-
-              p0 = NULL;
-              while ((length = es_read_line (verfp, &line, &line_len, NULL))>0)
-                {
-                  lnr++;
-                  trim_spaces (line);
-                  if (lnr == 1 && *line != '[')
-                    {
-                      /* Old file format where we look only at the
-                       * first line.  */
-                      p0 = line;
-                      break;
-                    }
-                  else if (!strncmp (line, "version=", 8))
-                    {
-                      p0 = line + 8;
-                      break;
-                    }
-                }
-              if (length < 0 || es_ferror (verfp))
-                es_fprintf (fp, "%s[VERSION file read error]\n", prefix);
-              else if (p0)
-                {
-                  for (p=p0; *p; p++)
-                    if (*p < ' ' || *p > '~' || *p == '[')
-                      *p = '?';
-                  es_fprintf (fp, "%s%s\n", prefix, p0);
-                }
-              else
-                es_fprintf (fp, "%s[VERSION file is empty]\n", prefix);
-
-              es_fclose (verfp);
-            }
-        }
-      xfree (fname);
-    }
-  xfree (line);
-
+              gpgrt_strusage (13), BUILD_REVISION, prefix, gpgrt_strusage (17));
 #ifdef HAVE_W32_SYSTEM
   {
     OSVERSIONINFO osvi = { sizeof (osvi) };
@@ -1263,7 +1204,7 @@ show_versions (estream_t fp)
 
 /* Copy data from file SRC to DST.  Returns 0 on success or an error
  * code on failure.  If LISTP is not NULL, that strlist is updated
- * with the variable or registry key names detected.  Flag bit 0
+ * with the variabale or registry key names detected.  Flag bit 0
  * indicates a registry entry.  */
 static gpg_error_t
 my_copy_file (estream_t src, estream_t dst, strlist_t *listp)
@@ -1276,14 +1217,7 @@ my_copy_file (estream_t src, estream_t dst, strlist_t *listp)
 
   while ((length = es_read_line (src, &line, &line_len, NULL)) > 0)
     {
-      /* Prefix each line with two spaces but use a comma if the line
-       * starts with a special org-mode character.  */
-      if (*line == '*' || (*line == '#' && line[1] == '+'))
-        es_fputc (',', dst);
-      else
-        es_fputc (' ', dst);
-      es_fputc (' ', dst);
-
+      /* Strip newline and carriage return, if present.  */
       written = gpgrt_fwrite (line, 1, length, dst);
       if (written != length)
 	return gpg_error_from_syserror ();
@@ -1351,19 +1285,21 @@ show_configs_one_file (const char *fname, int global, estream_t outfp,
   if (!fp)
     {
       err = gpg_error_from_syserror ();
-      if (gpg_err_code (err) != GPG_ERR_ENOENT)
-        es_fprintf (outfp, "** %s config \"%s\": %s\n",
-                    global? "global":"local", fname, gpg_strerror (err));
+      es_fprintf (outfp, "###\n### %s config \"%s\": %s\n###\n",
+                  global? "global":"local", fname,
+                  (gpg_err_code (err) == GPG_ERR_ENOENT)?
+                  "not installed" : gpg_strerror (err));
     }
   else
     {
-      es_fprintf (outfp, "** %s config \"%s\"\n#+begin_src\n",
+      es_fprintf (outfp, "###\n### %s config \"%s\"\n###\n",
                   global? "global":"local", fname);
+      es_fprintf (outfp, CUTLINE_FMT, "start");
       err = my_copy_file (fp, outfp, listp);
-      es_fprintf (outfp, "\n#+end_src\n");
       if (err)
-        log_error ("Error copying file \"%s\": %s\n",
+        log_error ("error copying file \"%s\": %s\n",
                    fname, gpg_strerror (err));
+      es_fprintf (outfp, CUTLINE_FMT, "end--");
       es_fclose (fp);
     }
 }
@@ -1380,13 +1316,11 @@ show_other_registry_entries (estream_t outfp)
     const char *name;
   } names[] =
   {
+    { 1, "HKLM\\Software\\Gpg4win:Install Directory" },
     { 1, "HKLM\\Software\\Gpg4win:Desktop-Version" },
     { 1, "HKLM\\Software\\Gpg4win:VS-Desktop-Version" },
-    { 1, "\\Software\\Gpg4win:Install Directory" },
-    { 1, "\\Software\\GnuPG:Install Directory" },
     { 1, "\\" GNUPG_REGISTRY_DIR ":HomeDir" },
-    { 1, "\\" GNUPG_REGISTRY_DIR ":DefaultLogFile" },
-    { 2, "\\Software\\Microsoft\\Office\\Outlook\\Addins\\GNU.GpgOL"
+    { 2, "Software\\Microsoft\\Office\\Outlook\\Addins\\GNU.GpgOL"
       ":LoadBehavior" },
     { 2, "HKCU\\Software\\Microsoft\\Office\\16.0\\Outlook\\Options\\Mail:"
       "ReadAsPlain" },
@@ -1418,7 +1352,6 @@ show_other_registry_entries (estream_t outfp)
   int group = 0;
   char *namebuf = NULL;
   const char *name;
-  int from_hklm;
 
   for (idx=0; (name = names[idx].name); idx++)
     {
@@ -1432,14 +1365,14 @@ show_other_registry_entries (estream_t outfp)
           name = namebuf;
         }
 
-      value = read_w32_reg_string (name, &from_hklm);
+      value = read_w32_reg_string (name);
       if (!value)
         continue;
 
       if (names[idx].group != group)
         {
           group = names[idx].group;
-          es_fprintf (outfp, "\n%s related:\n",
+          es_fprintf (outfp, "###\n### %s related:\n",
                       group == 1 ? "GnuPG Desktop" :
                       group == 2 ? "Outlook" :
                       group == 3 ? "\\Software\\GNU\\GpgOL"
@@ -1447,87 +1380,21 @@ show_other_registry_entries (estream_t outfp)
         }
 
       if (group == 3)
-        es_fprintf (outfp, "  %s=%s%s\n", names[idx].name, value,
-                    from_hklm? " [hklm]":"");
+        es_fprintf (outfp, "### %s=%s\n", names[idx].name, value);
       else
-        es_fprintf (outfp, "  %s\n    ->%s<-%s\n", name, value,
-                    from_hklm? " [hklm]":"");
+        es_fprintf (outfp, "### %s\n###   ->%s<-\n", name, value);
+
+      /* FIXME: We may want to add an indiction whethe found via HKLM
+       * or HKCU.  */
 
       xfree (value);
     }
 
+  es_fprintf (outfp, "###\n");
   xfree (namebuf);
 }
-
-
-/* Print registry entries take from a configuration file.  */
-static void
-show_registry_entries_from_file (estream_t outfp)
-{
-  gpg_error_t err;
-  char *fname;
-  estream_t fp;
-  char *line = NULL;
-  size_t length_of_line = 0;
-  size_t  maxlen;
-  ssize_t len;
-  char *value = NULL;
-  int from_hklm;
-  int any = 0;
-
-  fname = make_filename (gnupg_datadir (), "gpgconf.rnames", NULL);
-  fp = es_fopen (fname, "r");
-  if (!fp)
-    {
-      err = gpg_error_from_syserror ();
-      if (gpg_err_code (err) != GPG_ERR_ENOENT)
-        log_error ("error opening '%s': %s\n", fname, gpg_strerror (err));
-      goto leave;
-    }
-
-  maxlen = 2048; /* Set limit.  */
-  while ((len = es_read_line (fp, &line, &length_of_line, &maxlen)) > 0)
-    {
-      if (!maxlen)
-        {
-          err = gpg_error (GPG_ERR_LINE_TOO_LONG);
-          log_error ("error reading '%s': %s\n", fname, gpg_strerror (err));
-          goto leave;
-        }
-      trim_spaces (line);
-      if (*line == '#')
-        continue;
-
-      xfree (value);
-      value = read_w32_reg_string (line, &from_hklm);
-      if (!value)
-        continue;
-
-      if (!any)
-        {
-          any = 1;
-          es_fprintf (outfp, "Taken from gpgconf.rnames:\n");
-        }
-
-      es_fprintf (outfp, "  %s\n    ->%s<-%s\n", line, value,
-                  from_hklm? " [hklm]":"");
-
-    }
-  if (len < 0 || es_ferror (fp))
-    {
-      err = gpg_error_from_syserror ();
-      log_error ("error reading '%s': %s\n", fname, gpg_strerror (err));
-    }
-
- leave:
-  xfree (value);
-  xfree (line);
-  es_fclose (fp);
-  xfree (fname);
-}
 #endif /*HAVE_W32_SYSTEM*/
 
-
 /* Show all config files.  */
 static void
 show_configs (estream_t outfp)
@@ -1535,61 +1402,26 @@ show_configs (estream_t outfp)
   static const char *names[] = { "common.conf", "gpg-agent.conf",
                                  "scdaemon.conf", "dirmngr.conf",
                                  "gpg.conf", "gpgsm.conf" };
-  static const char *envvars[] = { "PATH",
-                                   "http_proxy", "HTTP_PROXY",
-                                   "https_proxy", "HTTPS_PROXY",
-                                   "LD_LIBRARY_PATH", "LD_PRELOAD",
-                                   "LD_AUDIT", "LD_ORIGIN_PATH" };
   gpg_error_t err;
   int idx;
   char *fname;
   gnupg_dir_t dir;
   gnupg_dirent_t dir_entry;
   size_t n;
-  int any, anywarn;
+  int any;
   strlist_t list = NULL;
   strlist_t sl;
   const char *s;
-  int got_gpgconfconf = 0;
 
-  es_fprintf (outfp, "# gpgconf -X invoked %s%*s-*- org -*-\n\n",
-              isotimestamp (time (NULL)), 28, "");
-  es_fprintf (outfp, "* General information\n");
-  es_fprintf (outfp, "** Versions\n");
-  show_version_gnupg (outfp, "  ");
-  es_fprintf (outfp, "  Libgcrypt %s\n", gcry_check_version (NULL));
-  es_fprintf (outfp, "  GpgRT %s\n", gpg_error_check_version (NULL));
-#ifdef HAVE_W32_SYSTEM
-  es_fprintf (outfp, "  Codepages:");
-  if (GetConsoleCP () != GetConsoleOutputCP ())
-    es_fprintf (outfp, " %u/%u", GetConsoleCP (), GetConsoleOutputCP ());
-  else
-    es_fprintf (outfp, " %u", GetConsoleCP ());
-  es_fprintf (outfp, " %u", GetACP ());
-  es_fprintf (outfp, " %u\n", GetOEMCP ());
-#endif
-  es_fprintf (outfp, "\n\n");
+  es_fprintf (outfp, "### Dump of all standard config files\n");
+  show_version_gnupg (outfp, "### ");
+  es_fprintf (outfp, "### Libgcrypt %s\n", gcry_check_version (NULL));
+  es_fprintf (outfp, "### GpgRT %s\n", gpg_error_check_version (NULL));
+  es_fprintf (outfp, "###\n\n");
 
-  es_fprintf (outfp, "** Directories\n");
   list_dirs (outfp, NULL, 1);
   es_fprintf (outfp, "\n");
 
-  es_fprintf (outfp, "** Environment\n#+begin_example\n");
-  for (idx=0; idx < DIM(envvars); idx++)
-    if ((s = getenv (envvars[idx])))
-      es_fprintf (outfp, "%s=%s\n", envvars[idx], s);
-  es_fprintf (outfp, "#+end_example\n");
-
-  es_fprintf (outfp, "* Config files\n");
-  fname = make_filename (gnupg_sysconfdir (), "gpgconf.conf", NULL);
-  if (!gnupg_access (fname, F_OK))
-    {
-      got_gpgconfconf = 1;
-      show_configs_one_file (fname, 1, outfp, &list);
-      es_fprintf (outfp, "\n");
-    }
-  xfree (fname);
-
   for (idx = 0; idx < DIM (names); idx++)
     {
       fname = make_filename (gnupg_sysconfdir (), names[idx], NULL);
@@ -1602,7 +1434,6 @@ show_configs (estream_t outfp)
     }
 
   /* Print the encountered registry values and envvars.  */
-  es_fprintf (outfp, "* Other info\n");
   if (list)
     {
       any = 0;
@@ -1613,21 +1444,20 @@ show_configs (estream_t outfp)
               {
                 any = 1;
                 es_fprintf (outfp,
-                            "** List of encountered environment variables\n"
-                            "#+begin_example\n");
+                            "###\n"
+                            "### List of encountered environment variables:\n");
               }
             if ((s = getenv (sl->d)))
-              es_fprintf (outfp, "   %-12s ->%s<-\n", sl->d, s);
+              es_fprintf (outfp, "### %-12s ->%s<-\n", sl->d, s);
             else
-              es_fprintf (outfp, "   %-12s [not set]\n", sl->d);
+              es_fprintf (outfp, "### %-12s [not set]\n", sl->d);
           }
       if (any)
-        es_fprintf (outfp, "#+end_example\n");
+        es_fprintf (outfp, "###\n");
     }
 
 #ifdef HAVE_W32_SYSTEM
-  es_fprintf (outfp, "** Registry entries\n");
-  es_fprintf (outfp, "#+begin_example\n");
+  es_fprintf (outfp, "###\n### Registry entries:\n");
   any = 0;
   if (list)
     {
@@ -1635,38 +1465,26 @@ show_configs (estream_t outfp)
         if ((sl->flags & 1))
           {
             char *p;
-            int from_hklm;
 
             if (!any)
               {
                 any = 1;
-                es_fprintf (outfp, "Encountered in config files:\n");
+                es_fprintf (outfp, "###\n### Encountered in config files:\n");
               }
-            if ((p = read_w32_reg_string (sl->d, &from_hklm)))
-              es_fprintf (outfp, "  %s ->%s<-%s\n", sl->d, p,
-                          from_hklm? " [hklm]":"");
+            if ((p = read_w32_reg_string (sl->d)))
+              es_fprintf (outfp, "### %s ->%s<-\n", sl->d, p);
             else
-              es_fprintf (outfp, "  %s [not set]\n", sl->d);
+              es_fprintf (outfp, "### %s [not set]\n", sl->d);
             xfree (p);
           }
     }
+  if (!any)
+    es_fprintf (outfp, "###\n");
   show_other_registry_entries (outfp);
-  show_registry_entries_from_file (outfp);
-  es_fprintf (outfp, "#+end_example\n");
 #endif /*HAVE_W32_SYSTEM*/
 
   free_strlist (list);
 
-  /* Additional warning.  */
-  anywarn = 0;
-  if (got_gpgconfconf)
-    {
-      anywarn = 1;
-      es_fprintf (outfp, "* Warnings\n");
-      es_fprintf (outfp,
-                  "- Legacy config file \"gpgconf.conf\" found\n");
-    }
-
   /* Check for uncommon files in the home directory.  */
   dir = gnupg_opendir (gnupg_homedir ());
   if (!dir)
@@ -1687,22 +1505,19 @@ show_configs (estream_t outfp)
               && dir_entry->d_name[n] == '-'
               && ascii_strncasecmp (dir_entry->d_name, "gpg.conf-1", 10))
             {
-              if (!anywarn)
-                {
-                  anywarn = 1;
-                  es_fprintf (outfp, "* Warnings\n");
-                }
               if (!any)
                 {
                   any = 1;
                   es_fprintf (outfp,
-                              "- Suspicious files in \"%s\":\n",
+                              "###\n"
+                              "### Warning: suspicious files in \"%s\":\n",
                               gnupg_homedir ());
                 }
-              es_fprintf (outfp, "  - %s\n", dir_entry->d_name);
+              es_fprintf (outfp, "### %s\n", dir_entry->d_name);
             }
         }
     }
+  if (any)
+    es_fprintf (outfp, "###\n");
   gnupg_closedir (dir);
-  es_fprintf (outfp, "# eof #\n");
 }
